package japflash;
import java.text.ParseException;
import java.util.Date;
import java.text.DateFormat;
/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */

public class FlashCard {
  public String kanji;
  public String reading;
  public String meaning;
  public int errorHistory;
  public int views;
  public long lastViewed;

  static final char RIGHT = 'R';
  static final char WRONG = 'W';
  static final char UNDEFINED = 'U';



/* The line has following rules:
 Kanji [Reading] /Meaning/ #ErrorHistory Date
*/
  public FlashCard(String c) throws ParseException{
    int brOpen = c.indexOf('[');
    int brClose = c.indexOf(']');
    int slash = c.indexOf('/',brClose);
    int lastSlash = c.lastIndexOf('/');
    int hash = c.indexOf('#',lastSlash);
    boolean hasReading = (brOpen > 0);
    boolean hasErrorHistory = (hash > lastSlash);
    int kanjiEnd = (hasReading)?brOpen:slash;


    if ((kanjiEnd < 1) ||
        (slash == -1)||
        (hasReading &&
        ((brClose  == -1)||(brClose <= brOpen) ))) throw
          new ParseException("Did not understand:"+c,0);

    kanji = c.substring(0,kanjiEnd).trim();
    if (kanji.length() == 0) throw
      new ParseException("Line:"+c+" does not define a kanji.",0);
    if (hasReading) {
      reading = c.substring(brOpen+1,brClose).trim();
 //     if (reading.length() == 0) throw
 //       new ParseException("Line:"+c+" does not define a reading.",0);
     if (reading.length() == 0) reading = null;

    }
    meaning = c.substring(slash+1,lastSlash).trim();
    if (meaning.length() == 0) throw
      new ParseException("Line:"+c+" does not define a meaning.",0);
    if (hasErrorHistory) {
        for (views = 0; ((views+hash+1) < c.length()) && (
                            (c.charAt(hash+1+views) == RIGHT)||
                            (c.charAt(hash+1+views) == WRONG)); views++) {
           errorHistory <<= 1;
           if (c.charAt(hash+1+views) == RIGHT) errorHistory += 1;
        }
    }
    if (c.indexOf('-',hash) != -1) {
      try {
        lastViewed = Integer.parseInt(c.substring(c.indexOf('-')));
      } catch (NumberFormatException e) {
      }
    }
  }

  public boolean isCorrect(int pos) {
    return (((errorHistory >> pos)&1)==1 );
  }

  public boolean historyIsDefined(int pos) {
    return (views>pos);
  }

  public boolean hasReading() {
    return (reading != null);
  }

  public void gotRight() {
    errorHistory <<= 1;
    errorHistory += 1;
    views++;
  }
  public void gotWrong() {
    views++;
    errorHistory <<= 1;
  }

  public boolean lastTwoCorrect() {
    if (views > 1) return isCorrect(0);
    if (views > 1) return (isCorrect(0)&&isCorrect(1));
    return false;
  }

  public void viewed() {
    lastViewed = System.currentTimeMillis();
  }

  public String getContent() {   // reads the cards content
    StringBuffer c = new StringBuffer();
    c.append(kanji);
    if (reading != null) c.append(" [" + reading + "]");
    c.append(" /" + meaning +"/ #");
    for (int i = 0; i < views; i++ ) c.append(isCorrect(i)?RIGHT:WRONG);
    if (lastViewed != 0) c.append("-"+lastViewed);
    return c.toString();
  }

}
