package japflash;

import java.awt.FileDialog;
import java.awt.*;
import java.awt.event.*;
import java.text.ParseException;
import java.io.*;
import java.util.StringTokenizer;
import java.awt.Toolkit;
import japsupport.*;
import java.util.Vector;
import tools.FieldInitialiser;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


public class JapFlash extends  Frame implements ItemListener{
  public Options o;
  Button closeButton = new Button();
  Button saveButton = new Button();
  Button loadButton = new Button();
  Button methodButton = new Button();
  Panel centerPanel = new Panel();
  CardLayout centerPanelLayout = new CardLayout();
  StackPanel sP;
  Label statusLabel = new Label();
  Panel statusBar = new Panel();
  Panel toolBar = new Panel();
  FlashCard card;
  FileDialog fileChooser = new FileDialog(this,"Save Lesson",FileDialog.SAVE);

  String iniFile = null;
  FileSelectorPanel fSP;

  public JapFlash(String[] args) {
    super("JAPFLASH");
    try {
      iniFile = System.getProperty("IniFile");
    } catch (Exception e) {e.printStackTrace();}
    o = new Options();
    if (iniFile  == null) {
      tools.LXToolBox.message("No ini file name specified, intialising default values");
    } else {
      try { FieldInitialiser.loadFields(o,iniFile);}
      catch (IOException e) {
        tools.LXToolBox.message("Failed to load ini file:"+iniFile+
         " therefore creating it using default values.");
        try {
          FieldInitialiser.saveFields(o,iniFile,o.iniFileHeader);
        } catch (IOException ioe) { ioe.printStackTrace();}
      }
    }
    sP = new StackPanel(o);
    if (o.loadLastFileAtStartup && (o.fileName.length() > 0))
      loadLesson(new File(o.fileName));
    fSP = new FileSelectorPanel(o,this);
    init();
  }

  void init() {
    setSize(240,300);
    closeButton.setLabel("Close");
    loadButton.setLabel("Load");
    saveButton.setLabel("Save as");
    methodButton.setLabel(FlashCardPrinter.TESTMODENAME[o.testMode]);

    centerPanel.add(sP);
    centerPanel.add(fSP);
    centerPanelLayout.addLayoutComponent(sP,"Stack");
    centerPanelLayout.addLayoutComponent(fSP,"FileSelector");
    centerPanel.setLayout(centerPanelLayout);
    add(centerPanel,BorderLayout.CENTER);

    toolBar.setLayout(new GridLayout(1,5));
    toolBar.add(methodButton);
    toolBar.add(loadButton);
    toolBar.add(saveButton);
    toolBar.add(closeButton);
    statusLabel.setAlignment(Label.LEFT);
    statusBar.setLayout(new GridLayout(1,1));
    statusBar.setBackground(Color.lightGray);
    statusBar.add(statusLabel);
    add(toolBar,BorderLayout.NORTH);
    add(statusBar,BorderLayout.SOUTH);
    tools.LXToolBox.setMessageLabel(statusLabel);
    fSP.setItemListener(this);
    if (sP.lesson != null) showStackPanel(); else showLessonChooserPanel();
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        quitApplication();
      }
      public void windowOpened(WindowEvent e) {
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(d.width/2-o.defaultScreenWidth/2,
                    d.height/2-o.defaultScreenHeight/2);
        setSize(o.defaultScreenWidth,o.defaultScreenHeight);
      };
    });
    methodButton.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e) {methodButtonPressed(e);}
      }
    );
    loadButton.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e) {loadButtonPressed(e);}
      }
    );
    saveButton.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e) {saveButtonPressed(e);}
      }
    );
    closeButton.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e) {closeButtonPressed();}
      }
    );
  }

  void methodButtonPressed(ActionEvent e) {
    if (o.testMode == FlashCardPrinter.ENGTOKANJI) {
      o.testMode = FlashCardPrinter.KANJITOENG;
    } else {
      o.testMode = FlashCardPrinter.ENGTOKANJI;
    }
    methodButton.setLabel(FlashCardPrinter.TESTMODENAME[o.testMode]);
  }

  void loadButtonPressed(ActionEvent e) {

   if ((sP.lesson != null) && !sP.lesson.cardStack.isEmpty()){
    YesNoCancelDialog closeDialog = new YesNoCancelDialog(
       this,"","Save file before loading new one?","Yes","No","Cancel");
    sP.putCardBack();
    closeDialog.show();
      switch (closeDialog.result) {
        case YesNoCancelDialog.CANCEL :
          return;
        case YesNoCancelDialog.NO :
          break;
        case YesNoCancelDialog.YES : {
          saveLesson(new File(o.fileName));
        }
      }
    }
    close();
    showLessonChooserPanel();
  }

  public void showStackPanel() {
    closeButton.setEnabled(true);
    loadButton.setEnabled(true);
    saveButton.setEnabled(true);
    methodButton.setEnabled(true);
    centerPanelLayout.show(centerPanel,"Stack");
    validate();
    repaint();
  }

  public void showLessonChooserPanel() {
    closeButton.setEnabled(false);
    loadButton.setEnabled(false);
    saveButton.setEnabled(false);
    methodButton.setEnabled(false);
    centerPanelLayout.show(centerPanel,"FileSelector");
    validate();
    repaint();
  }

  void saveButtonPressed(ActionEvent e) {
    if ((o.fileName != null)&&(o.fileName.length() != 0)) {
      fileChooser.setDirectory(o.fileName);
      fileChooser.setName(o.fileName);
    }
    fileChooser.show();
    if (fileChooser.getFile() != null)
      saveLesson(new File(fileChooser.getDirectory(),fileChooser.getFile()));
  }

  void closeButtonPressed() {
    //Custom button text
    YesNoCancelDialog closeDialog = new YesNoCancelDialog(
       this,"","Save file before closing?","Yes","No","Cancel");
    if (!sP.lesson.cardStack.isEmpty()){
    sP.putCardBack();
    closeDialog.show();
      switch (closeDialog.result) {
        case YesNoCancelDialog.CANCEL :
          sP.repaint();
          return;
        case YesNoCancelDialog.NO :
          close();
          sP.repaint();
          return;
        case YesNoCancelDialog.YES : {
          saveLesson(new File(o.fileName));
          sP.repaint();
          return;
        }
      }
    }
  }

  void quitApplication() {
    //Custom button text
    if ((sP.lesson != null)&&(!sP.lesson.cardStack.isEmpty())){
    sP.putCardBack();
    YesNoCancelDialog closeDialog = new YesNoCancelDialog(
       this,"","Save file before exiting?","Yes","No","Cancel");
    closeDialog.show();
      switch (closeDialog.result) {
        case YesNoCancelDialog.CANCEL : return;
        case YesNoCancelDialog.NO : close(); sP.repaint(); break;
        case YesNoCancelDialog.YES :
          saveLesson(new File(o.fileName));
          sP.repaint();
          break;
      }
    }

    try {
      FieldInitialiser.saveFields(o,iniFile,o.iniFileHeader);
    } catch (Exception e) {
      tools.LXToolBox.message("Could not save ini file because:");
      tools.LXToolBox.message(e.getMessage());
    }
    setVisible(false);
    dispose();
    System.exit(0);
  }

  public void close() {
    if (sP.lesson == null) return;
    if (sP.liftedCard != null) sP.putCardBack();
    sP.lesson.cardStack.clearStack();
    sP.lesson = null;
    o.fileName = "";              // no file is open at the moment
    this.setTitle(o.frameTitle);
  }

  public void itemStateChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
      LessonDescriptor ld = (LessonDescriptor)e.getItem();
      showStackPanel();
      tools.LXToolBox.debugMessage("Selected"+ld.file);
      loadLesson(ld.file);
    } else {
    showStackPanel();
    }
  }

  void loadLesson(File file){
    System.out.println("before loading o.filepath"+o.fileName);
    o.fileName = file.getAbsolutePath();
    System.out.println("after loading o.filepath"+o.fileName);
    sP.lesson = new Lesson(file,o);
    if (sP.lesson != null)
      setTitle(o.fileName);
    else
      setTitle("");
    sP.repaint();
  }

  void saveLesson(File file) {
    o.fileName = file.getAbsolutePath();
    System.out.println("o.filename"+o.fileName);
    sP.lesson.saveLesson(file);
    if (sP.lesson != null)
      setTitle(o.fileName);
    else
      setTitle("");
    sP.repaint();
  }

  public static void main(String[] args) {
    JapFlash app = new JapFlash(args);
    app.show();
  }
}