package japflash;

import java.awt.*;
import java.util.Vector;
import java.awt.event.*;


/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


  class SelectorCanvas extends Canvas {

    Graphics offScreenGraphics;
    Image offScreenImage;

    Vector items;
    Rectangle selectedItemBox;
    SelectorCanvasItemPainter iP;
    SelectionListener selectionListener;
    final static int SELECTIONDONE = 1;
    final static int SELECTIONCANCELED = 0;

    int scrollPosition;
    int selected = -1;
    int canvasSize;
    int itemBoxHeight= 30;
    int itemBoxHInset = 5;
    int itemBoxesOnScreen =1;

    public SelectorCanvas() {
      super();
      items = new Vector();

      selectedItemBox = new Rectangle(0,0,0,0);
      addMouseListener(new MouseAdapter() {
         public void mouseClicked(MouseEvent e) {
           checkMouseClick(e);
         };
      });

      addComponentListener(new ComponentAdapter() {
        public void componentResized(ComponentEvent e) {
        makeOffScreenGraphicsAndImage();
        repaint();
        }
      });
    };

    public void addItem(Object o) {
      items.addElement(o);
      repaint();
    }

    public boolean removeItem(Object o) {
      boolean result;
      result = items.removeElement(o);
      if (result) repaint();
      return result;
    }

    public boolean scrollDown() {
      if (items.size()>(scrollPosition+itemBoxesOnScreen-1)) {
        scrollPosition++;
        repaint();
        return true;
      }
      return false;
    }

    public boolean scrollUp() {
      if (scrollPosition > 0) {
        scrollPosition--;
        repaint();
        return true;
      }
      return false;
    }

    public void setSelectorCanvasItemPainter(SelectorCanvasItemPainter iP) {
      this.iP = iP;
    }

    public void setSelectionListener(SelectionListener sl) {
      this.selectionListener = sl;
    }

    public void setItemBoxSize(int itemBoxHInset,int itemBoxHeight) {
      this.itemBoxHInset = itemBoxHInset;
      this.itemBoxHeight = itemBoxHeight;
    }

    public void update(Graphics g) {
      paint(g);
    }

    public void paint(Graphics g) {
      if (offScreenGraphics == null) makeOffScreenGraphicsAndImage();
      offScreenGraphics.setClip(0,0,getSize().width,getSize().height);
      offScreenGraphics.clearRect(0,0,getSize().width,getSize().height);
      drawSelectorCanvas(offScreenGraphics);
      g.drawImage(offScreenImage,0,0,this);
//      g.clearRect(0,0,getSize().width,getSize().height);
//      drawSelectorCanvas(g);
    }

    public void makeOffScreenGraphicsAndImage() {
      offScreenImage = createImage(getSize().width,getSize().height);
      offScreenGraphics = offScreenImage.getGraphics();
    }

    public void drawSelectorCanvas(Graphics g ) {
      Rectangle box;
      int w = this.getSize().width;
      int h = this.getSize().height;

      if ((iP != null) && (!items.isEmpty())) {
        for (int i=0; i< items.size(); i++) {
          box = getItemBox(i,i == selected);
          g.setClip(box.x,box.y,box.width,box.height);
          g.setColor(Color.lightGray);
          g.fillRect(box.x,box.y,box.width,box.height);
          iP.drawItem(items.elementAt(i),g,box);
        }
        if (selected >= 0) {
          box = getItemBox(selected,true);
          g.setClip(box.x,box.y,box.width,box.height);
          g.setColor(Color.yellow);
          g.fillRect(box.x,box.y,box.width,box.height);
          iP.drawItemDetail(items.elementAt(selected),g,box);
        }
      }
    }

    public void select(int index) {
      selected = index;
      repaint();
    }
    public void selectionDone() {
      System.out.println("Selection Done:"+selected+" is "+
       ((LessonDescriptor)items.elementAt(selected)).file  );
      if (selectionListener != null)
        selectionListener.itemSelected(items.elementAt(selected));
    }
    public void selectionCanceled() {
      selectionListener.selectionCanceled();
    }

    private void checkMouseClick(MouseEvent e) {
      int hitBox = checkClickedBoxes(e.getPoint());
      if (selected == -1) {
        if (hitBox != -1) select(hitBox);
      } else {
        if (hitBox == selected ) selectionDone();
        else select(hitBox);
      }
    }

    private int checkClickedBoxes(Point p) {
      p.translate(0,scrollPosition);
      if (selectedItemBox.contains(p)) return -2;
      int hitBox = items.size()-1;
      while  ((hitBox>=0) &&
              ! (getItemBox(hitBox,selected == hitBox)).contains(p))
        hitBox--;
      return hitBox;
    }

    private Rectangle getItemBox(int index,boolean selected) {
      int gap;
      int excess;
      int h = getSize().height;
      int w = getSize().width;
      itemBoxesOnScreen = (h/itemBoxHeight)+1;
      while (true) {
        excess = h - (itemBoxHeight* itemBoxesOnScreen);
        gap = excess/itemBoxesOnScreen;
        if ((gap >5)||(itemBoxesOnScreen == 1)) break;
        itemBoxesOnScreen--;
      };
      if (!selected) {
        return new Rectangle(
          itemBoxHInset,
          (index-scrollPosition)*(itemBoxHeight+gap) +
            ((itemBoxesOnScreen == 1)?gap/2:(itemBoxHeight+gap)/2),
          w - itemBoxHInset - itemBoxHInset,
          itemBoxHeight);
      } else {
        return new Rectangle(
          itemBoxHInset-2,
          (index-scrollPosition)*(itemBoxHeight+gap) +
            ((itemBoxesOnScreen == 1)?gap/2:(itemBoxHeight+gap)/2)-
            itemBoxHeight/2,
          w - itemBoxHInset - itemBoxHInset+4,
          itemBoxHeight*2);
      }
    }

  }


  interface SelectorCanvasItemPainter {
    public void drawItemDetail(Object o, Graphics g,Rectangle r);
    public void drawItem(Object o, Graphics g,Rectangle r);
  }

  interface SelectionListener {
    public void itemSelected(Object o);
    public void selectionCanceled();
  }