package japsupport;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */

public class BitMapTextCanvas extends TextCanvas {
  static final int LEFT = 0;
  static final int CENTER = 1;
  static final int RIGHT = 2;
  static final int inset = 4;
  Image images[];
  BitMapFont font;
  int bytesPerChar;
  int numChars;
  int justification;

  public BitMapTextCanvas(BitMapFont font, int bytesPerChar, int numChars) {
    super();
    this.font = font;
    this.bytesPerChar = bytesPerChar;
    this.numChars = numChars;
    setChar(0);
  }

  public BitMapTextCanvas(BitMapFont font, int charNum, int bytesPerChar,
			  int numChars) {
    super();
    this.font = font;
    this.bytesPerChar = bytesPerChar;
    this.numChars = numChars;
    setChar(charNum);
    justification = LEFT;
  }

  public BitMapTextCanvas(BitMapFont font, byte bytes[], int bytesPerChar,
			  int numChars) {
    super();
    this.font = font;
    this.bytesPerChar = bytesPerChar;
    this.numChars = numChars;
    setChar(bytes);
    justification = LEFT;
  }

  public void setJustification (int justification) {
    this.justification = justification;
  }

  public void setChar(int charNum) {
    images = new Image[1];
    images[0] = font.getChar(charNum);
    repaint();
  }

  public void setChar(byte bytes[]) {
    try {
      int numChars = bytes.length / bytesPerChar;
      images = new Image[numChars];
      byte buf[] = new byte[bytesPerChar];
      for (int i=0; i<numChars; i++) {
	System.arraycopy(bytes, i*bytesPerChar, buf, 0, bytesPerChar);
	images[i] = font.getChar(buf);
      }
      repaint();
    }
    catch (NullPointerException npe) {
      setChar(0);
    }
  }

  public void setText(String text) {
    try {
      // put code to conver string to font indexes
      final int BYTESPERCHAR = 2;
      ByteArrayOutputStream byteOut = new ByteArrayOutputStream(text.length()*BYTESPERCHAR);
      OutputStreamWriter out = new OutputStreamWriter(byteOut, "JIS");
      out.write(text, 0, text.length()); //write string to array of JIS bytes in byteOut
      out.flush();
      ByteArrayInputStream byteIn = new ByteArrayInputStream(byteOut.toByteArray());
      JISInputStream in = new JISInputStream(byteIn);
      Vector v = new Vector(text.length());
      try {
	while(true) {
	  byte[] bytes = in.readBytes();
	  if (bytes != null)
	    v.addElement(bytes);
	  else
	    break;
	}
      }
      catch (NullPointerException npe) {
	// do nothing -- signals end of text
      }
      int totalBytes = 0;
      for (int i = 0; i < v.size(); i++) {
	totalBytes += ((byte[])(v.elementAt(i))).length;
      }
      byte[] bytes = new byte[totalBytes];
      int curPos = 0;
      for (int i = 0; i < v.size(); i++) {
	byte[] buff = (byte[])v.elementAt(i);
	System.arraycopy(buff, 0, bytes, curPos, buff.length);
	curPos += buff.length;
      }
      setChar(bytes);
      super.setText(text);
    }
    catch(IOException ioe) {
      setChar(0);
      this.text = "";
    }
  }

  public void paint(Graphics g) {
    try {
      if (showing) {
	double charWidth = 0.0;
	Dimension d = null;
	if (numChars != -1) {
	  //d = preferredSize();
	  d = getSize();
	  d.width -= 2 * inset;
	  d.height -= 2 * inset;
	  charWidth = (double) d.width / (double) numChars;
	}
	else {
	  d = getSize();
	  d.width -= 2 * inset;
	  d.height -= 2 * inset;
	  charWidth = (double) d.width / (double) images.length;
	}
	int charSize = Math.min((int) charWidth, d.height);

	double charPos = (double) inset;
	if (justification == CENTER)
	  charPos += (double) (Math.abs(numChars)*((int)charWidth - charSize)/2);
	else if (justification == RIGHT)
	  charPos += (double) (Math.abs(numChars)*((int)charWidth - charSize));

	int topOffset = inset + (d.height - charSize)/2;
	for (int i=0; i<images.length; i++) {
	  g.drawImage(images[i], (int) charPos, topOffset, charSize, charSize,
		      (ImageObserver) this);
	  charPos += charSize;
	}
      }
      else {
	Dimension d = getSize();
	g.setColor(getBackground());
	g.fillRect(0, 0, d.width, d.height);
      }
    }
    catch (NullPointerException npe) {}
    super.paint(g);
  }


  public boolean imageUpdate(Image img, int infoflags, int x, int y,
			     int width, int height) {
    if ((infoflags & ImageObserver.ALLBITS) != 0) {
      repaint();
    }
    return true;
  }
}

